/*
 *  Copyright (c) 2021 Honeywell International, Inc. All rights reserved.
 *
 *  This file contains trade secrets of Honeywell International, Inc. No part
 *  may be reproduced or transmitted in any form by any means or for any
 *  purpose without the express written permission of Honeywell.
 */


define([
    'baja!',
    'nmodule/webEditors/rc/fe/baja/BaseEditor',
    'bajaux/mixin/subscriberMixIn',
    'bajaux/util/SaveCommand',
    'bajaScript/baja/ord/SlotPath',
    'Promise',
    'hbs!nmodule/honIrmControl/rc/templates/typeCharacteristicParFe',
    'lex!honIrmControl'
    ], 
    function(      //NOSONAR
        baja,
        BaseEditor,
        subscriberMixIn,
        SaveCommand,
        SlotPath,
        Promise,
        typeCharacteristicParFe,
        lex
    ) {
    'use strict';

    var honIrmControlLex = lex[0];

    var TypeCharacteristicParFe = function () { // NOSONAR
        BaseEditor.apply(this, arguments);
        this.$component = undefined;
        subscriberMixIn(this);
        this.getCommandGroup().add(new SaveCommand());
    };


    TypeCharacteristicParFe.prototype = Object.create(BaseEditor.prototype);
    TypeCharacteristicParFe.prototype.constructor = TypeCharacteristicParFe;

    /**
     * Handle the initialization of the html template and register a changed event handler to set Modified flag and
     * handle the update of the inCharacteristicPar select element
     * @param {JQuery} dom 
     */
    TypeCharacteristicParFe.prototype.doInitialize = function(dom) {
        var that = this;
        dom.html(typeCharacteristicParFe({
            "label_masterSyncEnabled": honIrmControlLex.get("typecharacteristicPar.fe.label.masterSyncEnabled") || "Master Sync Enabled",
            "label_inTypePar": honIrmControlLex.get("typecharacteristicPar.fe.label.typePar") || "In Type Par",
            "label_inCharacteristicTypePar": honIrmControlLex.get("typecharacteristicPar.fe.label.characteristicPar") || "In Characteristic Par",
            "masterSyncEnabledlabel": "true",
            "cell_padding": "5px",
        }));
        dom.find('select').on('change', function() {
            that.setModified(true);
        });
        dom.find('input').on('change', function() {
            that.setModified(true);
        });

        dom.find('select#inTypeParSelect').on('change', function() {
            var inTypeParSelect = dom.find('select#inTypeParSelect');
            var inTypeParValue = inTypeParSelect.val();
            var oridnalInteger = baja.$('baja:Integer').make(inTypeParValue);

            that.$component.getTypeBasedCharacteristic(oridnalInteger).then(function(characteristicEnum) {
                that.updateInCharacteristicPar(characteristicEnum, characteristicEnum.getRange());
            });
        });
        
        dom.find('input#masterSyncEnabledInput').on('change', function() {
            var masterSyncEnabledInput = dom.find('input#masterSyncEnabledInput');
            var masterSyncEnabled = masterSyncEnabledInput.is(':checked');
            dom.find('label.masterSyncEnabledlabel').text(masterSyncEnabled);
        });
    };

    /**
     * Loads the initial values from the component and updates all the UI elements. 
     * Also register the `changed` event at the component subscriber.
     * 
     * @param {baja:Component} comp 
     */
    TypeCharacteristicParFe.prototype.doLoad = function(comp) {
        var that = this;
        this.$component = comp;
       	var dom = this.jq();
        
        
        // need to lease because after learn from controller the enum don't provide enough information yet.
        Promise.all([comp.$parent.lease(3), comp.lease(2), comp.loadSlots()]).then(function() {

            var masterSyncEnabled = comp.getMasterSyncEnabled();

            // update masterSyncEnabled.
            that.updateMasterSyncEnabled(masterSyncEnabled);

            var inTypePar = comp.getInTypePar();
            var inTypeParRange = inTypePar.getRange();
            comp.rpc("getInTypeParRange").then(function(result) {
                if (inTypeParRange.getOrdinals().length === 0) {
                    inTypeParRange = result;
                }
                that.updateInTypePar(inTypePar, inTypeParRange);
            });	
       
            var inCharacteristicPar = comp.getInCharacteristicPar();
            var inCharacteristicParRange = inCharacteristicPar.getRange();
            comp.rpc("getInCharacteristicParRange").then(function(result) {
                if (inCharacteristicParRange.getOrdinals().length === 0) {
                    inCharacteristicParRange = result;
                }
                that.updateInCharacteristicPar(inCharacteristicPar, inCharacteristicParRange);
            });
        });

		 comp.rpc("getFlagsForTypeParProps").then(function(result) {
			if(result == "true")
			{
			dom.find('input#masterSyncEnabledInput').attr('disabled', 'disabled');
			dom.find('#inTypeParSelect').prop('disabled', true);
			dom.find('#inCharacteristicParSelect').prop('disabled', true);
			}
			});

        this.getSubscriber().attach('changed', function(property, context) {
            var propName = property.getName();

            if (propName === 'masterSyncEnabled') {
                that.updateMasterSyncEnabled(property.$val);
            }
            else if (propName === 'inTypePar') {
                that.updateInTypePar(property.$val, property.$val.getRange());
            }
            else if (propName === 'inCharacteristicPar') {
                that.updateInCharacteristicPar(property.$val, property.$val.getRange());
            }
        });
    };

    /**
     * Handle the save of the selected user values to the component.
     * Will use the updateTypeAndCharacteristicFromUxFE Action form the BTypeCharacteristicPar Component.
     * @param {baja:Component} readValue 
     * @param {any} params 
     */
    TypeCharacteristicParFe.prototype.doSave = function(readValue, params) {
        var jq = this.jq();

        var masterSyncEnabledInput = jq.find('input#masterSyncEnabledInput');
        var inTypeParSelect = jq.find("select#inTypeParSelect");
        var inCharacteristicParSelect = jq.find("select#inCharacteristicParSelect");
        
        var inTypeParEnum = this.select2DynamicEnum(inTypeParSelect);
        var inCharacteristicParEnum = this.select2DynamicEnum(inCharacteristicParSelect);

        var toUpdate = baja.$("baja:Component");

        toUpdate.add({
            slot: "masterSyncEnabled", 
            value: baja.$("baja:Boolean").make(masterSyncEnabledInput.is(':checked'))
        });
        toUpdate.add({
            slot: "inTypePar",
            value: inTypeParEnum
        });
        toUpdate.add({
            slot: "inCharacteristicPar",
            value: inCharacteristicParEnum
        });

        readValue.updateTypeAndCharacteristicFromUxFE(toUpdate);
    };
    
    /**
     * Handles the detach of the subscribers on destroy.
     */
    TypeCharacteristicParFe.prototype.doDestroy = function() {
        this.getSubscriber().detach('changed');
    };

    /**
     * convenient function to update the masterSyncEnabled select element. 
     * @param {boolean} masterSyncEnabled 
     */
    TypeCharacteristicParFe.prototype.updateMasterSyncEnabled = function(masterSyncEnabled) {
        var jq = this.jq();
        var masterSyncEnabledInput = jq.find('input#masterSyncEnabledInput');
        masterSyncEnabledInput.prop('checked', masterSyncEnabled);
    };

    /**
     * convenient function to update the inTypePar select element. 
     * @param {BDynamicEnum} inTypeParEnum 
     */
    TypeCharacteristicParFe.prototype.updateInTypePar = function(inTypeParEnum, range) {
        var jq = this.jq();
        var inTypeParSelect = jq.find("select#inTypeParSelect");

        //cleanup before updating.
        inTypeParSelect.empty();
        var currentOrdinal = parseInt(inTypeParEnum.getOrdinal());
        for (var ordinal in range.$byOrdinal) {
            var itemText = range.getDisplayTag(parseInt(ordinal));
            if (currentOrdinal === parseInt(ordinal)) {
                inTypeParSelect.append('<option value="' + ordinal + '"selected>' +  itemText + "</option>");
            }
            else {
                inTypeParSelect.append('<option value="' + ordinal + '">' +  itemText + "</option>");
            }
        }
    };

    /**
     * convenient function to update the InCharacteristicPar select element. 
     * @param {BDynamicEnum} InCharacteristicParEnum 
     */
    TypeCharacteristicParFe.prototype.updateInCharacteristicPar = function(InCharacteristicParEnum, range) {
        var jq = this.jq();
        var inCharacteristicParSelect = jq.find("select#inCharacteristicParSelect");

        //cleanup before updating.
        inCharacteristicParSelect.empty();
        var currentOrdinal = parseInt(InCharacteristicParEnum.getOrdinal());
        for (var ordinal in range.$byOrdinal) {
            var itemText = range.getDisplayTag(parseInt(ordinal));
            if (currentOrdinal === parseInt(ordinal)) {
                inCharacteristicParSelect.append('<option value="' + ordinal + '"selected>' +  itemText + "</option>");
            }
            else {
                inCharacteristicParSelect.append('<option value="' + ordinal + '">' +  itemText + "</option>");
            }
        }
    };


    /**
     * convenient function to convert the options of the select element to a baja:DynamicEnum.
     * @param {JQuery} select 
     * @returns baja:DynamicEnum
     */
    TypeCharacteristicParFe.prototype.select2DynamicEnum = function(select) {
        var selectedOrdinal = parseInt(select.val());
        var options = select.find('option').toArray();
        var ordinals = options.map(function(option) { return option.value; });
        var tags = options.map(function(option) { return SlotPath.escape(option.text); });

        return baja.$("baja:DynamicEnum").make({
            ordinal: selectedOrdinal, 
            range: baja.$('baja:EnumRange').make({
                ordinals: ordinals,
                tags: tags 
            })
        });
    };

    return TypeCharacteristicParFe;
});